import os
import json
import logging
from concord.dcos_utils import *

CONCORD_FILENAME = '.concord.cfg'

CONCORD_DEFAULTS = { 'zookeeper_path' : '/concord',
                     'zookeeper_hosts' : 'localhost:2181' }

def build_logger(module_name, fmt_string=None):
    if fmt_string is not None:
        logging.basicConfig(format=fmt_string)
    else:
        logging.basicConfig()
    logger = logging.getLogger(module_name)
    level = logging.INFO if 'DCOS_LOG_LEVEL' not in os.environ else \
            os.environ['DCOS_LOG_LEVEL']
    logger.setLevel(level)
    return logger

def find_config(src, config_file):
    """ recursively searches .. until it finds a file named config_file
    will return None in the case of no matches or the abspath if found"""
    filepath = os.path.join(src, config_file)
    if os.path.isfile(filepath):
        return filepath
    elif src == '/':
        return None
    else:
        return find_config(os.path.dirname(src), config_file)

def fetch_config_opts(src, config_file):
    if ON_DCOS is True:
        return dcos_config_data()

    location = find_config(os.getcwd(), CONCORD_FILENAME)
    config_data = {}
    if location is None:
        config_data = config.CONCORD_DEFAULTS
    else:
        with open(location, 'r') as data_file:
            config_data = json.load(data_file)
    return config_data
    
def default_options(opts):
    config_data = fetch_config_opts(os.getcwd(), CONCORD_FILENAME)
    opts_methods = dir(opts)
    opts.zookeeper = config_data['zookeeper_hosts'] if opts.zookeeper is None \
                     else opts.zookeeper
    opts.zk_path =  config_data['zookeeper_path'] if opts.zk_path is None \
                    else opts.zk_path

def default_manifest_options(manifest, all_options):
    config_data = fetch_config_opts(os.getcwd(), CONCORD_FILENAME)
    for option, value in config_data.iteritems():
        if option in all_options and option not in manifest:
            manifest[option] = value
            
def human_readable_units(num, suffix='B'):
    for unit in ['','K','M','G','T','P','E','Z']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Y', suffix)



#!/usr/bin/env python
"""
Example of configuration file
{
    "executable_arguments": [
        "--hello=world",
        "--zookeeper=2",
        "logtostderr=2"
    ],
    "compress_files": [
        "folder",
        "file1.py",
        "file2.cpp"
    ],
    "fetch_url": "",
    "mem": 2048,
    "disk":2048,
    "executor_args" : ["--kafka_consumer_debug=cgrp"],
    "retries" : 3,
    "zookeeper_path": "/concord",
    "cpus": 4,
    "framework_v_module": "",
    "instances": 1,
    "framework_logging_level": 1,
    "environment_variables": [
        "LD_LIBRARY_PATH=./lib:/usr/local/lib",
        "MY_OPT=2"
    ],
    "zookeeper_hosts": "127.0.0.1:2181,10.1.2.3:2181",
    "executable_name": "my_binary",
    "exclude_compress_files": [
        ".git"
    ],
    "computation_name": "word-count-source",
    "update_binary": true,
    "execute_as_user": "agallego",
    "docker_container": "concord/client_devbox",
    "force_pull_container": "true"
}
"""

import os
import re
import json
import tarfile
import logging
import argparse
import uuid
from kazoo.client import KazooClient
from concord.internal.thrift.ttypes import *
from concord.utils import *
from concord.thrift_utils import *
from concord.functional_utils import *

logger = build_logger('cmd.deploy')

DEFAULTS = dict(
    mem = 256,
    update_binary = True,
    disk = 2048,
    cpus = 1,
    instances = 1,
    execute_as_user = "",
    environment_variables = [],
    executable_arguments = [],
    framework_v_module = "",
    framework_logging_level = 0,
    exclude_compress_files = [],
    docker_container = "",
    force_pull_container = True,
    retries = 0,
    executor_args = []
)

def validate_json_raw_config(dictionary, parser):
    valid_keys = ["executable_arguments", "docker_container",
                  "fetch_url", "mem", "disk", "cpus", "retries",
                  "framework_v_module", "instances",
                  "framework_logging_level", "environment_variables",
                  "zookeeper_hosts", "exclude_compress_files",
                  "update_binary", "execute_as_user",
                  "docker_container", "force_pull_container",
                  "executor_args"]
    reqs = ['compress_files', 'executable_name', 'computation_name',
            'zookeeper_hosts', 'zookeeper_path']

    # Add additional options from local concord.cfg file if necessary
    default_manifest_options(dictionary, valid_keys + reqs)

    all_keys = list(valid_keys)
    all_keys.extend(reqs)

    for k in dictionary:
        if k not in all_keys:
            parser.error("Key is not a valid concord request key: " + str(k))

    for k in reqs:
        if not dictionary.has_key(k):
            contents = json.dumps(dictionary, indent=4, separators=(',', ': '))
            parser.error("Please specify: " + k + ", parsed file: " + contents)


def parseFile(filename, parser):
    if os.path.splitext(filename)[1] != '.json':
        raise Exception('config file must end in .json')

    data = {}
    with open(filename) as data_file:
        data = json.load(data_file)

    validate_json_raw_config(data, parser)

    conf = DEFAULTS.copy()
    conf.update(data)

    if not conf["executable_name"] in conf["compress_files"]:
        parser.error("The executable name '" + conf["executable_name"] +
                     "' must exist in the 'compress_files' list")

    return conf

def generate_options():
    parser = argparse.ArgumentParser()
    parser.add_argument("config", metavar="config-file", action="store",
                        help="i.e: ./src/config.json")
    return parser

def validate_options(options, parser):
    if not options.config:
        parser.error("need to specify config file")

def tar_file_list(white_list, black_list):
    """
    Add files/dirs in the white_list. When you encounter something in the
    blacklist, exclude it. This is useful when adding a subdirectory containing
    many files, some of which you wish to exclude. The black_list should be
    formatted as regular expressions.
    """

    def expand_white_list(path):
        if os.path.isfile(path):
            return [path]
        files, directories = split_predicate(lambda x: os.path.isfile(x), os.listdir(path))
        return files + flat_map(lambda x: expand_white_list(path + '/' + x), directories)

    # set is useful for deduping
    white_list = flat_map(lambda x: expand_white_list(x), set(white_list))
    black_list = map(re.compile, set(black_list))

    # Return paths that don't contain any matches against any pieces of the path
    def exclude_matches(path):
        def on_black_list(component):
            return any(map(lambda x: x.match(component) != None, black_list))

        path_components = path.split('/')
        # We cannot guarantee that in mesos we'll have even read access to
        # parent dirs. So we cannot support compressing files wihtout flattening
        # the structure. Which means for simplicity instead of providing tuples
        # (../relative../../path -> desired/destination/path) PER file added
        # we only support files starting from this directory down
        # Same restriction as the one imposed by docker.
        #
        if len(path_components) > 0 and path_components[0] == '..':
            raise Exception("\n\n==> Relative paths that do not start from this %s" %
                         "directory are not supported, bad path: %s \n" % path)

        # Truncate by one if relative dir is prepended with a '.'
        if len(path_components) > 0 and path_components[0] == '.':
            path_components = path_components[1:-1]
        return not any(map(on_black_list, path_components))

    return filter(exclude_matches, white_list)

def build_thrift_request(request):
    logger.debug("JSON Request: %s" % json.dumps(request, indent=4, separators=(',', ': ')))
    tar_files = tar_file_list(request["compress_files"],
                              request["exclude_compress_files"])

    tar_name = os.path.abspath("concord_slug_" + str(uuid.uuid4()) + ".tar.gz")
    if os.path.exists(tar_name): logger.debug("%s exists! overriding" % tar_name)
    if len(tar_files) == 0: raise Exception("Nothing to tar.gz :'(")
    logger.debug("Created tar file: %s", tar_name)
    with tarfile.open(tar_name, "w:gz") as tar:
        for name in tar_files:
            logger.info("Adding tarfile: %s" % name)
            tar.add(name)

    logger.debug("Reading slug into memory")
    slug = open(tar_name, "rb").read()
    logger.info("Size of tar file is: %s", human_readable_units(len(slug)))
    if os.path.exists(tar_name):
        logger.debug("Removing slug %s", tar_name)
        os.remove(tar_name)

    logger.debug("Making request object")
    req = BoltComputationRequest()
    req.name = request["computation_name"]
    req.instances = request["instances"]
    req.cpus = request["cpus"]
    req.mem = request["mem"]
    req.disk = request["disk"]
    req.executorArgs = request["executor_args"]
    req.forceUpdateBinary = request["update_binary"]
    req.forcePullContainer = request["force_pull_container"]
    req.taskHelper = ExecutorTaskInfoHelper()
    req.taskHelper.retries = request["retries"]
    req.taskHelper.execName = request["executable_name"]
    req.taskHelper.clientArguments = request["executable_arguments"]
    req.taskHelper.environmentExtra = request["environment_variables"]
    req.taskHelper.frameworkLoggingLevel = request["framework_logging_level"]
    req.taskHelper.frameworkVModule = request["framework_v_module"]
    req.taskHelper.folder = os.path.dirname(
        os.path.relpath(request["executable_name"]))
    req.taskHelper.user = request["execute_as_user"]
    req.taskHelper.dockerContainer = request["docker_container"]
    logger.info("Thrift Request: %s" % thrift_to_json(req))
    req.slug = slug
    return req

def register(request, config):
    with ContextDirMgr(config):
        req = build_thrift_request(request)
        logger.debug("Getting master ip from zookeeper")
        ip = get_zookeeper_master_ip(
            request["zookeeper_hosts"], request["zookeeper_path"])
        (addr, port) = ip.split(":")

        logger.info("Sending computation to: %s" % ip)

        cli = get_sched_service_client(addr,int(port))
        logger.debug("Sending request to scheduler")
        cli.deployComputation(req)
        logger.debug("Done sending request to server")
        logger.info("Verify with the mesos host: %s that the service is running" % addr)

def main():
    parser = generate_options()
    options = parser.parse_args()
    validate_options(options,parser)
    register(parseFile(options.config, parser), options.config)

if __name__ == "__main__":
    main()



import os

class ContextDirMgr:
    """ Use with a 'with' keyword. All code in context will be in desired
    directory context"""
    def __init__(self, path):
        self.new_dir = os.path.dirname(os.path.abspath(path))

    def __enter__(self):
        self.old_dir = os.getcwd()
        os.chdir(self.new_dir)

    def __exit__(self, value, type, traceback):
        os.chdir(self.old_dir)

def flatten(xs):
    """ Flattens a nested list one level deep"""
    return reduce(lambda m, x: m + x, xs,[])

def flat_map(predicate, xs):
    """ Applys a transformation that transforms list into list(list), then
    flattens the results """
    return reduce(lambda memo, x: memo + x, map(predicate, xs), [])

def split_predicate(predicate, xs):
    """ Returns a tuple pair of lists where items from xs that match
    the predicate fall into the first list and the rest in the second."""
    def splitter(memo, x):
        first, second = memo
        if predicate(x):
            first.append(x)
        else:
            second.append(x)
        return (first, second)

    return reduce(splitter, xs, ([],[]))

def pairs_todict(kvpair_list):
    """ Transform list -> dict i.e. [a=b, c=d] -> {'a':'b', 'c':'d'}"""
    return None if kvpair_list is None else \
    { k:v for k, v in map(lambda c: c.split('='), kvpair_list) }




import json
import logging
from kazoo.client import KazooClient
from concord.internal.thrift.ttypes import *
from concord.internal.thrift import BoltSchedulerService
from concord.utils import build_logger
from thrift import Thrift
from thrift.protocol import (
    TJSONProtocol, TBinaryProtocol
)
from thrift.transport import (
    TSocket,TTransport
)

logging_format_string='%(levelname)s:%(asctime)s %(filename)s:%(lineno)d] %(message)s'
logger = build_logger('cmd.thrift_utils', logging_format_string)

def bytes_to_thrift(bytes, thrift_struct):
    transportIn = TTransport.TMemoryBuffer(bytes)
    protocolIn = TBinaryProtocol.TBinaryProtocol(transportIn)
    thrift_struct.read(protocolIn)

def thrift_to_json(thrift_struct):
    return json.dumps(thrift_struct, default=lambda o: o.__dict__,
            sort_keys=True, indent=4)

def get_zookeeper_master_ip(zkurl, zkpath):
    logger.info("Connecting to:%s" % zkurl)
    zk = KazooClient(hosts=zkurl)
    ip = ""
    try:
        logger.debug("Starting zk connection")
        zk.start()
        if not zk.exists(zkpath):
            logger.error('Path on zk doesn\'t exist')
            return ip
        logger.debug("Serializing TopologyMetadata() from %s" % zkpath)
        data, stat = zk.get(zkpath + "/masterip")
        logger.debug("Status of 'getting' %s/masterip: %s" % (zkpath, str(stat)))
        ip = str(data)
    except Exception as e:
        logger.exception(e)
    finally:
        logger.debug("Closing zk connection")
        zk.stop()
    return ip

def get_zookeeper_metadata(zkurl, zkpath):
    logger.info("Connecting to: %s" % zkurl)
    zk = KazooClient(hosts=zkurl)
    meta = TopologyMetadata()
    try:
        logger.debug("Starting zk connection")
        zk.start()
        if not zk.exists(zkpath):
            logger.error('Path on zk doesn\'t exist')
            return None
        logger.debug("Serializing TopologyMetadata() from %s" % zkpath)
        data, stat = zk.get(zkpath)
        logger.debug("Status of 'getting' %s: %s" % (zkpath, str(stat)))
        bytes_to_thrift(data, meta)
    except Exception as e:
        logger.exception(e)
        return None
    finally:
        logger.debug("Closing zk connection")
        zk.stop()

    return meta

def tproto(ip, port):
    socket = TSocket.TSocket(ip, port)
    transport = TTransport.TFramedTransport(socket)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    return (protocol, transport)

def get_sched_service_client(ip, port):
    (protocol, transport) = tproto(ip, port)
    client = BoltSchedulerService.Client(protocol)
    transport.open()
    return client





#!/usr/bin/env python
import os
import pip
from concord import constants
from setuptools import setup, find_packages
from pip.req import parse_requirements
from subprocess import call

reqs_file = os.path.join(os.path.dirname(os.path.realpath(__file__))
                   , "requirements.txt")

install_reqs = parse_requirements(reqs_file, session=pip.download.PipSession())

# reqs is a list of requirement
# e.g. ['django==1.5.1', 'mezzanine==1.4.6']
reqs = [str(ir.req) for ir in install_reqs]

setup(
    name='concord',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=constants.version,

    description='python concord command line tools',

    # Author details
    author='Concord Systems',
    author_email='support@concord.io',

    # Project homepage
    url='https://github.com/concord/cmd',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',

        # Pick your license as you wish (should match "license" above)
        # 'License :: OSI Approved :: TODO: License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7'
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax, for
    # example:
    # $ pip install -e .[dev,test]
    extras_require={},

    # What does your project relate to?
    keywords='dcos mesos streams operator',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'concord': [
            'resources/*'
        ]
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages.
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],
    data_files=[],

    # List run-time dependencies here.  These will be installed by pip when your
    # project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=reqs,

    # Directory where tests exist
    test_suite="tests",

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [ 'concord = concord.concord_entry:main' ]
        # on  dcos dcos-concord is needed otherwise its not wanted
    }
)



import os
import sys
import argparse
from dcos import util

ON_DCOS = True if 'DCOS_SSL_VERIFY' in os.environ else False

BASE_USAGE = """concord [-h] [--info] [--version] subcommand [suboptions [suboptions ...]]"""

POSITIONAL = """
\npositional arguments: (one of)
  deploy\tDeploy concord operators
  kill\t\tLaunch interactive session to browse and kill operators
  graph\t\tCreate a graphical representation of the current topology
  marathon\tCreate a marathon application from given parameters
  config\tSet global CLI defaults
"""

USAGE = BASE_USAGE + POSITIONAL

def dcos_options(parser):
    """Adds dcos specific options to the initial argument parser"""
    if ON_DCOS is not True:
        return
    parser.add_argument('--config-schema', action='store_true', help='Schema Validator')

def dcos_config_data():
    """Fetches config data from dcos schema"""
    config_data = {}
    config_data['zookeeper_hosts'] = util.get_config().get('concord.zookeeper_hosts')
    config_data['zookeeper_path'] = util.get_config().get('concord.zk_path')
    return config_data



import os
import time
import json
from argparse import ArgumentParser

DEFAULT_MARATHON = {
    "constraints": [ [ "hostname", "UNIQUE" ] ],
    "ports":[0],
    "upgradeStrategy": {
        "minimunHealthCapacity": 0.5,
        "maximumOverCapacity":0.5
    },
    "container": {
        "type": "DOCKER",
        "docker": {
            "image": "concord/scheduler:0.3.16.4",
            "network": "HOST"
        }
    }
}

DEFAULT_MARATHON_FILENAME = "concord_marathon_{0}.json".format(time.strftime("%H:%M:%S"))
DEFAULT_SERVICE_DISCOVERY = "$(/sbin/ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}')"
OPENDNS_SERVICE_DISCOVERY = "$(dig +short myip.opendns.com @resolver1.opendns.com)"

def generate_options():
    parser = ArgumentParser()
    parser.add_argument("-C", "--concord_zookeeper", default="zk://localhost:2181/concord"
                        ,help="i.e. zk://1.2.3.4:2181,2.2.2.2:2181/concord", action="store")
    parser.add_argument("-M", "--mesos_zookeeper", default="zk://localhost:2181/mesos"
                        ,help="i.e. zk://1.2.3.4:2181,2.2.2.2:2181/mesos", action="store")
    parser.add_argument("-n", "--framework_name"
                        ,help="Name to give to Concord Scheduler framework"
                        ,default="concord-scheduler", action="store")
    parser.add_argument("-l", "--locate_publicip", dest='locate', action='store_true',
                        help="""Use openDNS to automatically resolve public ip. 
                        If this is not set then ifconfig will be used to
                        query for a public ip. This may not work if the selected
                        machine is behind a router that uses NAT.""")
    parser.add_argument("-c", "--cpu_shares"
                        ,default=1, type=int, action="store")
    parser.add_argument("-m", "--mem_allocated"
                        ,default=1024, type=int, action="store")
    parser.add_argument("-i", "--instances", help="Instances of Concord Scheduler"
                        ,default=1, type=int, action="store")
    parser.add_argument("-o", "--output_destination"
                        ,default="./", action="store")
    parser.set_defaults(locate=False)
    return parser

def build_cmd(options):
    service_discovery = DEFAULT_SERVICE_DISCOVERY if options.locate is False \
                        else OPENDNS_SERVICE_DISCOVERY
    return ("java "
            "-XX:+UseConcMarkSweepGC "
            "-XX:+CMSParallelRemarkEnabled "
            "-XX:+HeapDumpOnOutOfMemoryError "
            "-XX:HeapDumpPath=/tmp/concord_heap_dump_{5}.hprof "
            "-Dsun.net.inetaddr.ttl=60 "
            "-Djava.library.path=/usr/lib "
            "-DMESOS_NATIVE_JAVA_LIBRARY=/usr/lib/libmesos.so "
            "$JAVA_OPTS -cp /usr/local/share/concord/concord.jar"
            " com.concord.scheduler.Scheduler "
            "--listen 0.0.0.0:$PORT0 "
            "--advertise-external {3} "
            "--advertise-internal {4} "
            "--concord-master {0} "
            "--mesos-master {1} "
            "--framework-name {2}").format(options.concord_zookeeper,
                                           options.mesos_zookeeper,
                                           options.framework_name.lower(),
                                           service_discovery,
                                           DEFAULT_SERVICE_DISCOVERY,
                                           time.strftime("%s"))

def build_marathon(options):
    # Assemble JSON file
    marathon_opts = DEFAULT_MARATHON.copy()
    marathon_opts["id"] = options.framework_name.lower()
    marathon_opts["cmd"] = build_cmd(options)
    marathon_opts["cpus"] = options.cpu_shares
    marathon_opts["mem"] = options.mem_allocated
    marathon_opts["instances"] = options.instances
    return json.dumps(marathon_opts, indent=2)

def write_marathon(marathon_json, dest_path, parser):
    basename = os.path.basename(dest_path)
    if '.' in basename:
        filename, ext = basename.split('.')
        if ext != 'json':
            parser.error("Output filename must end in '.json'")
    else:
        dest_path += DEFAULT_MARATHON_FILENAME

    with open(dest_path, 'w') as data_file:
        data_file.write(marathon_json)
        print "Successfully wrote Marathon file to: " + dest_path

def main():
    parser = generate_options()
    options = parser.parse_args()
    marathon_json = build_marathon(options)
    write_marathon(marathon_json, options.output_destination, parser)

if __name__ == "__main__":
    main()
    



#!/usr/bin/env python
import os #environ
import sys
import concord.kill_task, concord.print_graph
import concord.marathon, concord.deploy, concord.config
from concord import constants
from argparse import ArgumentParser
from pkg_resources import resource_string
from dcos_utils import *

def config_err():
    print "Cannot use 'concord config' command in DC/OS mode"

SUBPROGRAMS = {
    'config': concord.config.main if ON_DCOS is False else config_err,
    'kill': concord.kill_task.main,
    'graph': concord.print_graph.main,
    'marathon': concord.marathon.main,
    'deploy': concord.deploy.main    
}

def generate_options():
    parser = ArgumentParser(prog='concord', add_help=False, usage=USAGE)
    parser.add_argument('-h', '--help', action='store_true', help='Show this message')
    parser.add_argument('--info', action='store_true', help='Information about this cli')
    parser.add_argument('--version', action='store_true', help='Version info')
    dcos_options(parser)
    return parser

def run(options, program_options):
    # Pass along -h if detected but use after subcommand
    if options.help is True:
        program_options.append('-h')
        options.help = False

    # Attempt to run subcommand, passing along options
    program = program_options[0]
    if program in SUBPROGRAMS.keys():
        sys.argv = program_options
        SUBPROGRAMS[program]()
    else:
        print "'%s' is not a recognized command" % program

def main():
    parser = generate_options()
    options, program_options = parser.parse_known_args()

    # Remove 'concord' string if on DC/OS
    if ON_DCOS is True:
        program_options = program_options[1:]

    if options.info is True:
        print 'Deploy and manage Concord operators'
    elif options.version is True:
        print 'Concord CLI version %s' % constants.version
    elif ON_DCOS is True and options.config_schema is True:
        print resource_string(__name__, '/resources/config_schema.json')
    elif len(program_options) > 0:
        run(options, program_options)
    elif options.help is True:
        parser.print_help()

if __name__ == '__main__':
    main()
